/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.actionSystem.Presentation;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.fileEditor.FileDocumentManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.vfs.VirtualFileManager;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.*;


/**
 * Action to reformat from the project/package view.
 *
 * @version $Revision: 1.2 $
 */
public class BrowserFormatAction
    extends AnAction
{
    public void actionPerformed ( final AnActionEvent event )
    {
        ApplicationManager.getApplication (  ).runWriteAction ( new FormatWriteAction( event ) );
    }

    public void update ( AnActionEvent event )
    {
        final DataContext    dataContext   = event.getDataContext (  );
        final VirtualFile [] selectedFiles = ( VirtualFile [] ) dataContext.getData ( DataConstants.VIRTUAL_FILE_ARRAY );

        Presentation         presentation  = event.getPresentation (  );

        if ( ( selectedFiles == null ) || ( selectedFiles.length == 0 ) )
        {
            presentation.setEnabled ( false );
            presentation.setVisible ( false );
        }
        else if ( selectedFiles.length > 1 )
        {
            presentation.setEnabled ( true );
            presentation.setVisible ( true );
            presentation.setText ( "Jalopy selected files" );
        }
        else if ( IdeaUtil.isJavaFile ( selectedFiles [ 0 ] ) && IdeaUtil.isInProject ( selectedFiles [ 0 ], dataContext ) )
        {
            presentation.setEnabled ( true );
            presentation.setVisible ( true );
            presentation.setText ( "Jalopy '" + selectedFiles [ 0 ].getName (  ) + "'" );
        }
        else if ( selectedFiles [ 0 ].isDirectory (  ) )
        {
            presentation.setEnabled ( true );
            presentation.setVisible ( true );
            presentation.setText ( "Jalopy in '" + selectedFiles [ 0 ].getName (  ) + "'" );
        }
        else
        {
            presentation.setEnabled ( false );
            presentation.setVisible ( false );
        }
    }

    private static class FormatWriteAction
        implements Runnable
    {
        private final VirtualFile [] selectedFiles;
        private DataContext          dataContext;

        FormatWriteAction ( AnActionEvent event )
        {
            dataContext        = event.getDataContext (  );
            this.selectedFiles = ( VirtualFile [] ) dataContext.getData ( DataConstants.VIRTUAL_FILE_ARRAY );
        }

        public void run (  )
        {
            FileDocumentManager.getInstance (  ).saveAllDocuments (  );

            try
            {
                formatFiles (  );
            }
            catch ( Exception e )
            {
                StringWriter writer = new StringWriter(  );
                e.printStackTrace ( new PrintWriter( writer ) );
                Messages.showMessageDialog ( writer.toString (  ), "Error during jalopy !", new ImageIcon( getClass (  ).getResource ( "/jalopy.png" ) ) );
            }

            VirtualFileManager.getInstance (  ).refresh ( false );
        }

        private void formatFiles (  )
                           throws FileNotFoundException
        {
            formatFiles ( selectedFiles );
        }

        private void formatFiles ( final VirtualFile [] currentFiles )
                           throws FileNotFoundException
        {
            for ( final VirtualFile file : currentFiles )
            {
                if ( IdeaUtil.isJavaFile ( file ) && IdeaUtil.isInProject ( file, dataContext ) )
                {
                    JalopyReport report = JalopyPlugin.getInstance (  ).format ( file );
                    manageErrorAndWarning ( report );
                }
                else if ( file.isDirectory (  ) )
                {
                    formatFiles ( file.getChildren (  ) );
                }
            }
        }

        private void manageErrorAndWarning ( JalopyReport report )
        {
            if ( report.hasError (  ) )
            {
                String fileName = report.getFormatedFile (  ).getPresentableName (  );
                Messages.showErrorDialog ( report.buildMessage (  ), fileName + " : " + report.stateToString (  ) );
            }
            else if ( report.hasWarning (  ) )
            {
                String fileName = report.getFormatedFile (  ).getPresentableName (  );
                Messages.showWarningDialog ( report.buildMessage (  ), fileName + " : " + report.stateToString (  ) );
            }
        }
    }
}
