/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.fileTypes.FileType;
import com.intellij.openapi.fileTypes.FileTypeManager;
import com.intellij.openapi.localVcs.LocalVcs;
import com.intellij.openapi.localVcs.LvcsAction;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VirtualFile;

import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiFile;


/**
 * Contains Utilites for IDEA.
 */
final class IdeaUtil
{
    private IdeaUtil (  )
    {
    }

    public static Project getProject ( DataContext dataContext )
    {
        return ( Project ) dataContext.getData ( DataConstants.PROJECT );
    }

    public static LvcsAction lvcsStart ( DataContext  dataContext,
                                         final String actionInfo )
    {
        final LocalVcs lvcs        = LocalVcs.getInstance ( IdeaUtil.getProject ( dataContext ) );
        VirtualFile    currentFile = IdeaUtil.getCurrentFile ( dataContext );

        if ( ( currentFile == null ) || ( lvcs == null ) )
        {
            return LvcsAction.EMPTY;
        }

        return lvcs.startAction ( actionInfo, currentFile.getPath (  ), false );
    }

    public static void vcsLabel ( DataContext  dataContext,
                                  final String label )
    {
        LocalVcs.getInstance ( IdeaUtil.getProject ( dataContext ) ).addLabel ( label, getCurrentFile ( dataContext ).getPath (  ) );
    }

    public static VirtualFile getCurrentFile ( DataContext dataContext )
    {
        return ( VirtualFile ) dataContext.getData ( DataConstants.VIRTUAL_FILE );
    }

    public static PsiFile getPsiFile ( Editor        editor,
                                       final Project project )
    {
        Document document = editor.getDocument (  );

        return PsiDocumentManager.getInstance ( project ).getPsiFile ( document );
    }

    public static boolean isJavaFile ( DataContext dataContext,
                                       Editor      editor )
    {
        final VirtualFile currentFile = IdeaUtil.getCurrentFile ( dataContext );

        if ( currentFile != null )
        {
            return IdeaUtil.isJavaFile ( currentFile );
        }
        else
        {
            Project project = getProject ( dataContext );
            PsiFile psiFile = IdeaUtil.getPsiFile ( editor, project );

            return ( psiFile != null ) && isJavaFile ( psiFile.getVirtualFile (  ) );
        }
    }

    public static boolean isJavaFile ( VirtualFile currentFile )
    {
        // TODO[UGLY] Must Find the real way to determine if a file is a JAVA or not.
        final FileTypeManager typeMng          = FileTypeManager.getInstance (  );
        FileType              fileType         = typeMng.getFileTypeByFile ( currentFile );
        String                defaultExtension = fileType.getDefaultExtension (  );

        return defaultExtension.equalsIgnoreCase ( "java" );
    }

    public static boolean isInProject ( VirtualFile selectedFile,
                                        DataContext dataContext )
    {
        Project project       = getProject ( dataContext );
        Module  moduleForFile = VfsUtil.getModuleForFile ( project, selectedFile );

        return moduleForFile != null;
    }
}
