/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import org.jdom.Element;

import org.jetbrains.annotations.NotNull;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.NamedJDOMExternalizable;
import com.intellij.openapi.util.WriteExternalException;

import javax.swing.*;


/**
 * Jalopy plugin heart.
 */
public class JalopyComponent
    implements ApplicationComponent,
               Configurable,
               NamedJDOMExternalizable
{
    private static boolean      optimizeImport        = true;
    private static final String EXTERNALIZED_FILENAME = "jalopy-intellij";
    static final String         JALOPY_COMPONENT      = "JalopyComponent";
    private Logger              logger                = Logger.getInstance ( "idea.jalopy" );
    private PluginConfigPanel   configPanel;

    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "Component build" );
        }
    }

    public void disposeComponent (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "disposeComponent()" );
        }

        JalopyPlugin.dispose (  );
    }

    @NotNull
    public String getComponentName (  )
    {
        return JALOPY_COMPONENT;
    }

    public void initComponent (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "initComponent()" );
        }

        JalopyPlugin.init (  );
    }

    public String getDisplayName (  )
    {
        return "Jalopy for IDEA 6";
    }

    public Icon getIcon (  )
    {
        return new ImageIcon( getClass (  ).getResource ( "/jalopy.png" ) );
    }

    public String getHelpTopic (  )
    {
        return null;
    }

    public JComponent createComponent (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "createComponent()" );
        }

        configPanel                                   = new PluginConfigPanel(  );
        configPanel.setOptimizeImport ( optimizeImport );

        return configPanel;
    }

    public boolean isModified (  )
    {
        return ( configPanel != null ) && ( optimizeImport != configPanel.isOptimizeImport (  ) );
    }

    public void apply (  )
                throws ConfigurationException
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "apply()" );
        }

        if ( configPanel != null )
        {
            optimizeImport = configPanel.isOptimizeImport (  );
        }
    }

    public void reset (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "reset()" );
        }
    }

    public void disposeUIResources (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "disposeUIResources()" );
        }

        configPanel = null;
    }

    public String getExternalFileName (  )
    {
        return EXTERNALIZED_FILENAME;
    }

    public void readExternal ( Element element )
                       throws InvalidDataException
    {
        String optimizeStr = element.getTextTrim (  );

        optimizeImport = ! "not_OptimizeImport".equalsIgnoreCase ( optimizeStr );
    }

    public void writeExternal ( Element element )
                        throws WriteExternalException
    {
        element.addContent ( ( optimizeImport ? "optimizeImport" : "not_OptimizeImport" ) );
    }

    public static boolean isOptimizeimport (  )
    {
        return optimizeImport;
    }
}
