/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import org.apache.log4j.spi.LoggingEvent;

import com.intellij.openapi.vfs.VirtualFile;

import de.hunsicker.jalopy.Jalopy;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Report of a jalopy format (from one file or Editor).
 */
class JalopyReport
{
    private String             result       = null;
    private Jalopy.State       state        = Jalopy.State.OK;
    private VirtualFile        formatedFile;
    private List<LoggingEvent> eventList    = new ArrayList<LoggingEvent>(  );

    public VirtualFile getFormatedFile (  )
    {
        return formatedFile;
    }

    public String getResult (  )
    {
        return result;
    }

    public void setResult ( String result )
    {
        this.result = result;
    }

    public Jalopy.State getState (  )
    {
        return state;
    }

    public void setState ( Jalopy.State state )
    {
        this.state = state;
    }

    public void setFormatedFile ( VirtualFile formatedFile )
    {
        this.formatedFile = formatedFile;
    }

    public boolean hasError (  )
    {
        return getState (  ) == Jalopy.State.ERROR;
    }

    public boolean isOK (  )
    {
        return getState (  ) == Jalopy.State.OK;
    }

    public boolean hasWarning (  )
    {
        return getState (  ) == Jalopy.State.WARN;
    }

    public String buildMessage (  )
    {
        StringBuffer buffer = new StringBuffer(  );

        for ( LoggingEvent event : eventList )
        {
            appendInfo ( buffer, event );
        }

        return buffer.toString (  );
    }

    private void appendInfo ( StringBuffer buffer,
                              LoggingEvent event )
    {
        if ( event == null )
        {
            buffer.append ( "Unexpected error/warning without logging event !\n" );

            return;
        }

        if ( event.getThrowableInformation (  ) == null )
        {
            buffer.append ( "Unexpected error/warning !\n" ).append ( event.getMessage (  ) );

            return;
        }

        Throwable error    = event.getThrowableInformation (  ).getThrowable (  );
        String    errorMsg = error.toString (  );

        buffer.append ( event.getMessage (  ) );

        Pattern p = Pattern.compile ( "[0-9]+:[0-9]+" );
        Matcher m = p.matcher ( errorMsg );

        if ( m.find (  ) )
        {
            buffer.append ( '[' ).append ( m.group (  ) ).append ( ']' );
        }

        buffer.append ( ' ' );
        buffer.append ( error.getMessage (  ) ).append ( "\n" );
    }

    public String stateToString (  )
    {
        if ( getState (  ) == Jalopy.State.WARN )
        {
            return ( "Jalopy warnings !" );
        }
        else if ( getState (  ) == Jalopy.State.ERROR )
        {
            return ( "Jalopy error !" );
        }

        return "Ok";
    }

    public void addEvent ( LoggingEvent event )
    {
        eventList.add ( event );
    }
}
